@@ -10,9 +10,11 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
+import deprecation
 import mock
 import uuid
 
+from openstack import exceptions
 from openstack.network.v2 import _proxy
 from openstack.network.v2 import address_scope
 from openstack.network.v2 import agent
@@ -55,6 +57,7 @@ QOS_POLICY_ID = 'qos-policy-id-' + uuid.uuid4().hex
 QOS_RULE_ID = 'qos-rule-id-' + uuid.uuid4().hex
 NETWORK_ID = 'network-id-' + uuid.uuid4().hex
 AGENT_ID = 'agent-id-' + uuid.uuid4().hex
+ROUTER_ID = 'router-id-' + uuid.uuid4().hex
 
 
 class TestNetworkProxy(test_proxy_base2.TestProxyBase):
@@ -114,7 +117,7 @@ class TestNetworkProxy(test_proxy_base2.TestProxyBase):
     def test_dhcp_agent_hosting_networks(self):
         self.verify_list(
             self.proxy.dhcp_agent_hosting_networks,
-            agent.DHCPAgentHostingNetwork,
+            network.DHCPAgentHostingNetwork,
             paginated=False,
             method_kwargs={'agent': AGENT_ID},
             expected_kwargs={'agent_id': AGENT_ID}
@@ -123,7 +126,7 @@ class TestNetworkProxy(test_proxy_base2.TestProxyBase):
     def test_network_hosting_dhcp_agents(self):
         self.verify_list(
             self.proxy.network_hosting_dhcp_agents,
-            network.NetworkHostingDHCPAgent,
+            agent.NetworkHostingDHCPAgent,
             paginated=False,
             method_kwargs={'network': NETWORK_ID},
             expected_kwargs={'network_id': NETWORK_ID}
@@ -683,7 +686,8 @@ class TestNetworkProxy(test_proxy_base2.TestProxyBase):
                       self.proxy.get_quota_default,
                       method_args=['QUOTA_ID'],
                       expected_args=[quota.QuotaDefault],
-                      expected_kwargs={'project': "PROJECT"})
+                      expected_kwargs={'project': fake_quota.id,
+                                       'requires_id': False})
         mock_get.assert_called_once_with(quota.Quota, 'QUOTA_ID')
 
     def test_quotas(self):
@@ -739,6 +743,106 @@ class TestNetworkProxy(test_proxy_base2.TestProxyBase):
     def test_router_update(self):
         self.verify_update(self.proxy.update_router, router.Router)
 
+    @mock.patch.object(proxy_base2.BaseProxy, '_get_resource')
+    @mock.patch.object(router.Router, 'add_interface')
+    def test_add_interface_to_router_with_port(self, mock_add_interface,
+                                               mock_get):
+        x_router = router.Router.new(id="ROUTER_ID")
+        mock_get.return_value = x_router
+
+        self._verify("openstack.network.v2.router.Router.add_interface",
+                     self.proxy.add_interface_to_router,
+                     method_args=["FAKE_ROUTER"],
+                     method_kwargs={"port_id": "PORT"},
+                     expected_kwargs={"port_id": "PORT"})
+        mock_get.assert_called_once_with(router.Router, "FAKE_ROUTER")
+
+    @mock.patch.object(proxy_base2.BaseProxy, '_get_resource')
+    @mock.patch.object(router.Router, 'add_interface')
+    def test_add_interface_to_router_with_subnet(self, mock_add_interface,
+                                                 mock_get):
+        x_router = router.Router.new(id="ROUTER_ID")
+        mock_get.return_value = x_router
+
+        self._verify("openstack.network.v2.router.Router.add_interface",
+                     self.proxy.add_interface_to_router,
+                     method_args=["FAKE_ROUTER"],
+                     method_kwargs={"subnet_id": "SUBNET"},
+                     expected_kwargs={"subnet_id": "SUBNET"})
+        mock_get.assert_called_once_with(router.Router, "FAKE_ROUTER")
+
+    @mock.patch.object(proxy_base2.BaseProxy, '_get_resource')
+    @mock.patch.object(router.Router, 'remove_interface')
+    def test_remove_interface_from_router_with_port(self, mock_remove,
+                                                    mock_get):
+        x_router = router.Router.new(id="ROUTER_ID")
+        mock_get.return_value = x_router
+
+        self._verify("openstack.network.v2.router.Router.remove_interface",
+                     self.proxy.remove_interface_from_router,
+                     method_args=["FAKE_ROUTER"],
+                     method_kwargs={"port_id": "PORT"},
+                     expected_kwargs={"port_id": "PORT"})
+        mock_get.assert_called_once_with(router.Router, "FAKE_ROUTER")
+
+    @mock.patch.object(proxy_base2.BaseProxy, '_get_resource')
+    @mock.patch.object(router.Router, 'remove_interface')
+    def test_remove_interface_from_router_with_subnet(self, mock_remove,
+                                                      mock_get):
+        x_router = router.Router.new(id="ROUTER_ID")
+        mock_get.return_value = x_router
+
+        self._verify("openstack.network.v2.router.Router.remove_interface",
+                     self.proxy.remove_interface_from_router,
+                     method_args=["FAKE_ROUTER"],
+                     method_kwargs={"subnet_id": "SUBNET"},
+                     expected_kwargs={"subnet_id": "SUBNET"})
+        mock_get.assert_called_once_with(router.Router, "FAKE_ROUTER")
+
+    @mock.patch.object(proxy_base2.BaseProxy, '_get_resource')
+    @mock.patch.object(router.Router, 'add_gateway')
+    def test_add_gateway_to_router(self, mock_add, mock_get):
+        x_router = router.Router.new(id="ROUTER_ID")
+        mock_get.return_value = x_router
+
+        self._verify("openstack.network.v2.router.Router.add_gateway",
+                     self.proxy.add_gateway_to_router,
+                     method_args=["FAKE_ROUTER"],
+                     method_kwargs={"foo": "bar"},
+                     expected_kwargs={"foo": "bar"})
+        mock_get.assert_called_once_with(router.Router, "FAKE_ROUTER")
+
+    @mock.patch.object(proxy_base2.BaseProxy, '_get_resource')
+    @mock.patch.object(router.Router, 'remove_gateway')
+    def test_remove_gateway_from_router(self, mock_remove, mock_get):
+        x_router = router.Router.new(id="ROUTER_ID")
+        mock_get.return_value = x_router
+
+        self._verify("openstack.network.v2.router.Router.remove_gateway",
+                     self.proxy.remove_gateway_from_router,
+                     method_args=["FAKE_ROUTER"],
+                     method_kwargs={"foo": "bar"},
+                     expected_kwargs={"foo": "bar"})
+        mock_get.assert_called_once_with(router.Router, "FAKE_ROUTER")
+
+    def test_router_hosting_l3_agents_list(self):
+        self.verify_list(
+            self.proxy.routers_hosting_l3_agents,
+            agent.RouterL3Agent,
+            paginated=False,
+            method_kwargs={'router': ROUTER_ID},
+            expected_kwargs={'router_id': ROUTER_ID},
+        )
+
+    def test_agent_hosted_routers_list(self):
+        self.verify_list(
+            self.proxy.agent_hosted_routers,
+            router.L3AgentRouter,
+            paginated=False,
+            method_kwargs={'agent': AGENT_ID},
+            expected_kwargs={'agent_id': AGENT_ID},
+        )
+
     def test_security_group_create_attrs(self):
         self.verify_create(self.proxy.create_security_group,
                            security_group.SecurityGroup)
@@ -768,6 +872,7 @@ class TestNetworkProxy(test_proxy_base2.TestProxyBase):
         self.verify_update(self.proxy.update_security_group,
                            security_group.SecurityGroup)
 
+    @deprecation.fail_if_not_removed
     def test_security_group_open_port(self):
         mock_class = 'openstack.network.v2._proxy.Proxy'
         mock_method = mock_class + '.create_security_group_rule'
@@ -789,6 +894,7 @@ class TestNetworkProxy(test_proxy_base2.TestProxyBase):
             }
             mocked.assert_called_with(**expected_args)
 
+    @deprecation.fail_if_not_removed
     def test_security_group_allow_ping(self):
         mock_class = 'openstack.network.v2._proxy.Proxy'
         mock_method = mock_class + '.create_security_group_rule'
@@ -957,4 +1063,21 @@ class TestNetworkProxy(test_proxy_base2.TestProxyBase):
                         value=[mock.sentinel.project_id],
                         expected_args=[
                             auto_allocated_topology.ValidateTopology],
-                        expected_kwargs={"project": mock.sentinel.project_id})
+                        expected_kwargs={"project": mock.sentinel.project_id,
+                                         "requires_id": False})
+
+    def test_set_tags(self):
+        x_network = network.Network.new(id='NETWORK_ID')
+        self._verify('openstack.network.v2.network.Network.set_tags',
+                     self.proxy.set_tags,
+                     method_args=[x_network, ['TAG1', 'TAG2']],
+                     expected_args=[['TAG1', 'TAG2']],
+                     expected_result=mock.sentinel.result_set_tags)
+
+    @mock.patch('openstack.network.v2.network.Network.set_tags')
+    def test_set_tags_resource_without_tag_suport(self, mock_set_tags):
+        no_tag_resource = object()
+        self.assertRaises(exceptions.InvalidRequest,
+                          self.proxy.set_tags,
+                          no_tag_resource, ['TAG1', 'TAG2'])
+        self.assertEqual(0, mock_set_tags.call_count)
